import { world, GameMode } from "@minecraft/server";

const tags = [
  "minecraft:is_pickaxe",
  "minecraft:is_axe",
  "minecraft:is_shovel",
  "minecraft:is_hoe",
];

world.afterEvents.entityHitBlock.subscribe(({ damagingEntity, hitBlock }) => {
  if (getGamemode(damagingEntity) === "creative") return;
  const inv = damagingEntity.getComponent("inventory").container;
  const item = inv.getItem(damagingEntity.selectedSlot);

  if (tags.some((tag) => item?.hasTag(tag))) {
    correctTool(hitBlock, item, damagingEntity);
  }
});

function correctTool(block, mainHand, player) {
  const inv = player.getComponent("inventory").container;
  let requiredTool = "";

  const stoneTags = [
    "stone",
    "metal",
    "rail",
    "mob_spawner",
    "diamond_pick_diggable",
    "iron_pick_diggable",
  ];
  const woodTags = ["wood", "text_sign", "pumpkin"];
  const shovelTags = ["dirt", "sand", "gravel", "grass", "snow"];

  if (stoneTags.some((tag) => block.hasTag(tag))) {
    requiredTool = "minecraft:is_pickaxe";
  } else if (woodTags.some((tag) => block.hasTag(tag))) {
    requiredTool = "minecraft:is_axe";
  } else if (shovelTags.some((tag) => block.hasTag(tag))) {
    requiredTool = "minecraft:is_shovel";
  }

  if (requiredTool !== "") {
    if (mainHand && mainHand.hasTag(requiredTool)) return;
    const tools = [];

    for (let i = 0; i < inv.size; i++) {
      const item = inv.getItem(i);
      if (item && item.hasTag(requiredTool)) {
        const durability = item.getComponent("durability");
        const currentDurability = durability.maxDurability - durability.damage;

        tools.push({
          index: i,
          id: item.typeId,
          durability: currentDurability,
        });
      }
    }

    if (tools.length > 0) {
      tools.sort((a, b) => {
        if (a.durability !== b.durability) {
          return b.durability - a.durability;
        }
        return 0;
      });

      inv.swapItems(player.selectedSlot, tools[0].index, inv);
    }
  }
}

function getGamemode(player) {
  return Object.values(GameMode).find(
    (mode) =>
      [...world.getPlayers({ name: player.name, gameMode: mode })].length
  );
}
